//
//  SetCardView.m
//  Drawing
//
//  Created by Sameh Fakhouri on 10/29/14.
//  Copyright (c) 2014 Lehman College. All rights reserved.
//

#import "SetCardView.h"

@implementation SetCardView


#pragma mark - Properties

- (void)setSymbol:(NSString *)symbol
{
    _symbol = symbol;
    [self setNeedsDisplay];
}

- (void)setCount:(NSUInteger)count
{
    _count = count;
    [self setNeedsDisplay];
}

- (void)setColor:(UIColor *)color
{
    _color = color;
    [self setNeedsDisplay];
}

- (void)setShade:(NSString *)shade
{
    _shade = shade;
    [self setNeedsDisplay];
}

#pragma mark - Set Card Methods

+ (NSArray *)validSymbols
{
    return @[@"Oval", @"Diamond", @"Squiggle"];
}

+ (NSArray *)validShades
{
    return @[@"Solid", @"Striped", @"Outlined"];
}

+ (NSArray *)validColors
{
    return @[[UIColor greenColor], [UIColor purpleColor], [UIColor redColor]];
}

+ (NSUInteger)maxCount
{
    return 3;
}

#pragma mark - DrawRect

#define CORNER_FONT_STANDARD_HEIGHT 180.0
#define CORNER_RADIUS 12.0

- (CGFloat)cornerScaleFactor
{
    return self.bounds.size.height / CORNER_FONT_STANDARD_HEIGHT;
}

- (CGFloat)cornerRadius
{
    return CORNER_RADIUS * [self cornerScaleFactor];
}

- (void)drawRect:(CGRect)rect
{
    UIBezierPath *roundedRect = [UIBezierPath bezierPathWithRoundedRect:self.bounds cornerRadius:[self cornerRadius]];
    [roundedRect addClip];
    
    [[UIColor whiteColor] setFill];
    UIRectFill(self.bounds);
    
    [[UIColor blackColor] setStroke];
    [roundedRect stroke];
    
    [self drawDiamond];
}

#pragma mark - Draw Diamond

#define DIAMOND_UPPER_VERTEX_OFFSET 0.10
#define DIAMOND_LOWER_VERTEX_OFFSET 0.10
#define DIAMOND_WIDTH 0.20

- (CGPoint)upperVertex
{
    CGPoint upper = CGPointMake(self.bounds.size.width/2.0,
                                self.bounds.size.height * DIAMOND_UPPER_VERTEX_OFFSET);
    return upper;
}

- (CGPoint)lowerVertex
{
    CGPoint lower = CGPointMake(self.bounds.size.width/2.0,
                                self.bounds.size.height - (self.bounds.size.height * DIAMOND_LOWER_VERTEX_OFFSET));
    return lower;
}

- (CGPoint)leftVertex
{
    CGPoint left = CGPointMake(self.bounds.size.width/2.0 - self.bounds.size.width * DIAMOND_WIDTH / 2.0,
                               self.bounds.size.height / 2.0);
    return left;
}

- (CGPoint)rightVertex
{
    CGPoint right = CGPointMake(self.bounds.size.width/2.0 + self.bounds.size.width * DIAMOND_WIDTH / 2.0,
                                self.bounds.size.height / 2.0);
    return right;
}


- (void)drawDiamond
{
    UIBezierPath *diamond = [[UIBezierPath alloc] init];
    diamond.lineWidth = 2;
    
    [diamond moveToPoint:[self upperVertex]];
    [diamond addLineToPoint:[self rightVertex]];
    [diamond addLineToPoint:[self lowerVertex]];
    [diamond addLineToPoint:[self leftVertex]];
    [diamond addLineToPoint:[self upperVertex]];
    
    // set the color of the diamond
    [self.color setStroke];

    if (self.count == 1) {
        // count = 1
        // nothing to add to the drawing
    } else if (self.count == 2) {
        // count = 2
        // you need to make another diamond and position
        // the two of them to be centered
    } else {
        // count = 3
        UIBezierPath *leftDiamond = [diamond copy];
        CGAffineTransform leftTransform = {1.0, 0.0, 0.0, 1.0, self.bounds.size.width * 0.30 * -1.0, 0.0};
        [leftDiamond applyTransform:leftTransform];
        
        UIBezierPath *rightDiamond = [diamond copy];
        CGAffineTransform rightTransform = {1.0, 0.0, 0.0, 1.0, self.bounds.size.width * 0.30 , 0.0};
        [rightDiamond applyTransform:rightTransform];
        
        [diamond appendPath:leftDiamond];
        [diamond appendPath:rightDiamond];
    }
    
    if ([self.shade isEqualToString:[SetCardView validShades][0]]) {
        // It is colored solid
        [self.color setFill];
    } else if ([self.shade isEqualToString:[SetCardView validShades][1]]) {
        // It is stripped
        [[UIColor clearColor] setFill];
        CGPoint start = CGPointMake(0.0, 0.0);
        CGPoint end = CGPointMake(self.bounds.size.width, 0.0);
        CGFloat dy = self.bounds.size.height / 15.0;
        [diamond addClip];
        while (start.y <= self.bounds.size.height) {
            [diamond moveToPoint:start];
            [diamond addLineToPoint:end];
            start.y += dy;
            end.y += dy;
        }
    } else {
        // It must be outlined
        [[UIColor clearColor] setFill];
    }
    
    [diamond fill];
    [diamond stroke];
}

#pragma mark - Initialization

- (void)setup
{
    self.backgroundColor = nil;
    self.opaque = NO;
    self.contentMode = UIViewContentModeRedraw;
}


- (void)awakeFromNib
{
    [self setup];
}

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
    }
    return self;
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect
{
    // Drawing code
}
*/

@end
